@@ -1,5 +1,5 @@
 # -*-Perl-*-
-# @(#)DBlib.pm	1.31	12/30/97
+# @(#)DBlib.pm	1.34	11/06/98
 
 # Copyright (c) 1991-1997
 #   Michael Peppler
@@ -233,6 +233,7 @@ use vars qw(%Att);
 	     SYBFLT8 SYBIMAGE SYBINT1 SYBINT2 SYBINT4 SYBMONEY
 	     SYBMONEY4 SYBREAL SYBTEXT SYBVARBINARY SYBVARCHAR
 	     DBRPCRETURN DBRPCNORETURN DBRPCRECOMPILE
+	      DBRESULT DBNOTIFICATION DBINTERRUPT
 	      $DB_ERROR
 	     );
 
@@ -383,21 +384,26 @@ sub r_sql {
 #
 # Enhanced sql routine.
 # 
+
+sub DB_ERROR { return $DB_ERROR; }
+ 
+
 sub nsql {
     my ($db,$sql,$type) = @_;
     my (@res,@data,%data);
 
-    undef $DB_ERROR;
- 
-    unless ( $db->dbcmd($sql) ) {
-      $DB_ERROR = "Unable to submit SQL to Sybase server.";
-      return undef;
+    if ( ref $type ) {
+	$type = ref $type;
+    }
+    elsif ( not defined $type ) {
+	$type = "";
     }
 
-    unless ( $db->dbsqlexec ) {
-      $DB_ERROR = "Error executing SQL.";
-      return undef;
-    }
+    undef $DB_ERROR;
+ 
+    return unless $db->dbcmd($sql);
+  
+    return unless $db->dbsqlexec;
 
     while ( $db->dbresults != $db->NO_MORE_RESULTS ) {
       if ( ref $type eq "HASH" || $type eq "HASH" ) {
@@ -423,7 +429,7 @@ sub nsql {
     #
     # If we picked any sort of error, then don't feed the data back.
     #
-    return ( $DB_ERROR ? undef : @res );
+    return ( $DB_ERROR ? () : @res );
 
 }
 
@@ -431,12 +437,12 @@ sub nsql_error_handler {
     my ($db, $severity, $error, $os_error, $error_msg, $os_error_msg) = @_;
     # Check the error code to see if we should report this.
 
-    if ( $error != $db->SYBESMSG ) {
+    if ( $error != SYBESMSG ) {
       $DB_ERROR = "Sybase error: $error_msg\n";
       $DB_ERROR .= "OS Error: $os_error_msg\n" if defined $os_error_msg;
     }
 
-    $db->INT_CANCEL;
+    INT_CANCEL;
 }
 
 sub nsql_message_handler {
@@ -451,7 +457,10 @@ sub nsql_message_handler {
       $DB_ERROR .= "Line: $line\n" if defined $line;
       $DB_ERROR .= "Text: $text\n";
 
-      local ($lineno) = 1;
+      return unless ref $db;
+      
+      my ($lineno) = 1;
+      my $row;
       foreach $row ( split(/\n/,$db->dbstrcpy) ) {
           $DB_ERROR .= sprintf ("%5d", $lineno ++) . "> $row\n";
       }
