@@ -1,5 +1,5 @@
 # -*-Perl-*-
-# @(#)Sybperl.pm	1.25	06/27/97
+# @(#)Sybperl.pm	1.27	03/26/98
 #
 # Copyright (c) 1994-1995
 #   Michael Peppler
@@ -92,7 +92,7 @@ tie $dbBin0x, Sybase::Sybperl::Attribs, 'dbBin0x', 0;
 	     dbmny4cmp dbmny4sub dbmny4mul dbmny4minus dbmny4divide
 	     dbmny4add sql
 	     dbrpcinit dbrpcparam dbrpcsend
-	     dbwritetext);
+	     dbwritetext dbreadtext dbmoretext dbpreptext);
 
 
 # Internal routine to check that a parameter passed as $dbproc to one
@@ -220,6 +220,20 @@ sub dbsqlexec
     $ret;
 }
 
+sub dbsqlok
+{
+    my($dbproc) = @_;
+    my($ret);
+
+    if(!defined($dbproc) || !$dbproc || !&isadb($dbproc))
+    {
+	croak("It doesn't make sense to call dbsqlok with an undefined \$dbproc") if(!defined($default_db));
+	$dbproc = $default_db;
+    }
+    $ret = $dbproc->dbsqlok;
+    $ret;
+}
+
 sub dbresults
 {
     my($dbproc) = @_;
@@ -1114,6 +1128,74 @@ sub dbrpcinit
     $dbproc->dbrpcinit(@param);
 }
 
+sub dbreginit
+{
+    my(@param) = @_;
+    my($dbproc);
+
+    if(@param == 2)
+    {
+	$dbproc = shift(@param);
+    }
+    else
+    {
+	$dbproc = $default_db;
+    }
+
+    $dbproc->dbreginit(@param);
+}
+
+sub dbreglist
+{
+    my(@param) = @_;
+    my($dbproc);
+
+    if(@param == 1)
+    {
+	$dbproc = shift(@param);
+    }
+    else
+    {
+	$dbproc = $default_db;
+    }
+
+    $dbproc->dbreglist(@param);
+}
+
+sub dbregparam
+{
+    my(@param) = @_;
+    my($dbproc);
+
+    if(@param == 5)
+    {
+	$dbproc = shift(@param);
+    }
+    else
+    {
+	$dbproc = $default_db;
+    }
+
+    $dbproc->dbregparam(@param);
+}
+
+sub dbregexec
+{
+    my(@param) = @_;
+    my($dbproc);
+
+    if(@param == 2)
+    {
+	$dbproc = shift(@param);
+    }
+    else
+    {
+	$dbproc = $default_db;
+    }
+
+    $dbproc->dbregexec(@param);
+}
+
 sub dbwritetext
 {
     my(@params) = @_;
@@ -1124,6 +1206,37 @@ sub dbwritetext
     $dbproc->dbwritetext(@params);
 }
 
+sub dbreadtext
+{
+    my(@params) = @_;
+    my($dbproc);
+
+    $dbproc = shift(@params);
+
+    $dbproc->dbwritetext(@params);
+}
+
+sub dbmoretext
+{
+    my(@params) = @_;
+    my($dbproc);
+
+    $dbproc = shift(@params);
+
+    $dbproc->dbmoretext(@params);
+}
+
+sub dbpreptext
+{
+    my(@params) = @_;
+    my($dbproc);
+
+    $dbproc = shift(@params);
+
+    $dbproc->dbpreptext(@params);
+}
+
+
 1;
 
 
